export default class TaskQueue {
    #throttle;
    #asyncCallback = null;
    #items = new Set();
    #inRunLoop = false;
    constructor(asyncCallback, throttle) {
        this.#asyncCallback = asyncCallback;
        this.#throttle = throttle;
    }
    async #preRun() {
        if (this.#inRunLoop) {
            return;
        }
        this.#inRunLoop = true;
        if (!this.#throttle.leading) {
            await new Promise((resolve) => setTimeout(resolve, this.#throttle.wait));
        }
        await this.#run();
    }
    async #run() {
        if (this.#items.size === 0) {
            this.#inRunLoop = false;
            return;
        }
        this.#items.clear();
        if (this.#asyncCallback) {
            await this.#asyncCallback();
        }
        await new Promise((resolve) => setTimeout(resolve, this.#throttle.wait));
        await this.#run();
    }
    addMultiple(items) {
        for (const item of items) {
            this.#items.add(item);
        }
        if (this.#items.size > 0) {
            this.#preRun();
        }
    }
    add(item) {
        this.addMultiple([item]);
    }
}
//# sourceMappingURL=task-queue.js.map